function out=Dichotomy(Qdist,E,E2,Beta,NbrNeigh,Gamma)



%%  INPUT
%%%%%%%%%
%%  Qdist : matrix I x J x G
%%          Mean field approximate distribution
%%  E   : matrix 1 x Nbr
%%      see below the relation between S and the energy
%%  E2  : matrix 1 x Nbr
%%      (ln pi(z;Beta) +Cte) /(Beta) for the importance sampling.
%%  Beta : real
%%      Current value of the parameter
%%  NbrNeigh : integer
%%      Number of neighboors
%%  Gamma : positive number
%%      radius of the ball arounf the current value Beta, for the local optimization
%%
%%  OUTPUT
%%%%%%%%%%
%%  Structure with fields : 
%%  Beta:   Value of the optimum of the approximate quantity
%%  DiagCvg : Diagnostic of cvgce of the dichotomy procedure
%%          '1' if convergence.
%%  MeanQ : mean of H(z;b)/b wrt the Q-distribution
%%  GradWinit:  Gradient of the log of the approximate TildeW at the
%%          beginning of the oterative procedure
%%  GradWend:  Gradient of the log of the approximate TildeW at the
%%          end of the oterative procedure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          CONVENTION ON the POTTS MODEL 
%%
%%      ln p(z; b) =  H(z;b) - ln W(b)
%%      H(z;b) = 0.5 b \sum_k \sum_{j in N(k)} z_k' z_j
%%             = b \sum_{k ~ j} z_k' z_j
%%             = 0.5 b S(z) =  b E(z)
%%      b > 0.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%  1- PRE-SETTINGS
%%%%%%%%%%%%%%%%%%%%%
    Nbr=length(E);
    [I J G]=size(Qdist);
    Neigh=zeros(I,J);

    %% 1-1 Stopping rule for finding the root of the gradient iteratively
    TolGrad=10^(-8);  % tolerance on the gradient 
    Nmax=10000; % maximal number of iterations
    stop=0;    % stopping "bell"
    
    %% 1-2   Mean of the statistic S wrt the Qdist distribution
    QEnl=zeros(I+2,J+2,G);
    QEnl(2:I+1,2:J+1,:)=Qdist;
    QEnl(1,2:J+1,:)=Qdist(I,:,:);
    QEnl(I+2,2:J+1,:)=Qdist(1,:,:);
    QEnl(2:I+1,1,:)=Qdist(:,J,:);
    QEnl(2:I+1,J+2,:)=Qdist(:,1,:);
    QEnl(1,1,:)=Qdist(I,J,:);
    QEnl(I+2,J+2,:)=Qdist(1,1,:);
    QEnl(1,J+2,:)=Qdist(I,1,:);
    QEnl(I+2,1,:)=Qdist(1,J,:);
 
    switch NbrNeigh
        case '4-Neigh'
            Neigh=sum(QEnl(3:I+2,2:J+1,:).*QEnl(2:I+1,2:J+1,:),3);  % Lower
            Neigh=sum(QEnl(1:I,2:J+1,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh;  % Upper
            Neigh=sum(QEnl(2:I+1,3:J+2,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh; % Right
            Neigh=sum(QEnl(2:I+1,1:J,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh;  % Left
        case '8-Neigh'
            Neigh=sum(QEnl(3:I+2,2:J+1,:).*QEnl(2:I+1,2:J+1,:),3);  % Lower
            Neigh=sum(QEnl(1:I,2:J+1,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh;  % Upper
            Neigh=sum(QEnl(2:I+1,3:J+2,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh; % Right
            Neigh=sum(QEnl(2:I+1,1:J,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh;  % Left
            Neigh=sum(QEnl(1:I,1:J,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh;  % Upper-Left
            Neigh=sum(QEnl(1:I,3:J+2,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh;  % Upper-Right
            Neigh=sum(QEnl(3:I+2,1:J,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh;  % Lower-Left
            Neigh=sum(QEnl(3:I+2,3:J+2,:).*QEnl(2:I+1,2:J+1,:),3)+Neigh;  % Lower-Right
    end;
    
    %% 1-3 : conlude
    MeanQ=0.5*sum(sum(Neigh)); % this corresponds to : MeanQ = 0.5*sum_z S(z) Qdist(z) = sum_z E(z) Qdist(z)

    
%% 2 -  ITERATIVE PROCEDURE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%  2-1 : Initialization
    xOld=Beta;
    
    
    %%  2-2 : Gradient of the approximation of ln W(beta)
    Aux=xOld*E-Beta*E2; % matrix 1 x Nbr
    CtrAux=Aux-max(Aux);    % matrix 1 x Nbr
    SumAux=sum(exp(CtrAux));  % matrix 1 x 1
    ProbOld=exp(CtrAux)/SumAux;    % matrix 1 x Nbr
    if (round(sum(ProbOld))~=1),
        disp(sprintf('Numerical problem for the Beta-update part'));
    end;
    GradLnTildeW=E*ProbOld';
    
    %% 2-3 : Gradient of the objective function
    GradFctObjOld=MeanQ-GradLnTildeW;
    PlotGG(1)=GradFctObjOld;

    %% 2-4 : Compute the gradient in xOld +/- Gamma
    xNew=xOld+sign(GradFctObjOld)*Gamma;
    Aux=xNew*E-Beta*E2;
    CtrAux=Aux-max(Aux);
    ProbNew=exp(CtrAux)/sum(exp(CtrAux));    % matrix Nbr x 1
    if (round(sum(ProbNew))~=1),
        disp(sprintf('Numerical problem for the Beta-update part'));
    end;
    GradFctObjNew=MeanQ-E*ProbNew';
    
    %% 2-5 : Test if the root is in [xOld, xNew]
    if sign(GradFctObjNew)==sign(GradFctObjOld),
        %   'TRUE' means that the root of the gradient is too far from the
        %   initial point.
        out.Beta=xNew;
        out.DiagCvg=1;
        out.MeanQ=MeanQ;
        out.GradWinit=E*ProbOld';
        out.GradWend=E*ProbNew';
        %   Display
        figure(3);
        subplot(3,1,3);
        plot(ProbOld(1,:),'r:');
        hold on;
        plot(ProbNew(1,:),'g');
        title('weights');
        drawnow
        return;
        
    %% 2-6 : if 'YES', iterative procedure
    else
        %   'FALSE' means that the root is in the interval 
        %   [xOld-Gamma, xOld+Gamma]. Thus we can organize an iterative
        %   procedure for finding the root.
        nn=1;
        left=Beta-Gamma;
        right=Beta+Gamma;
        %   Gradient of the approximate quantity at the current value of Beta
        GradCurr=GradFctObjOld;
    
        while stop==0,
            xNew=0.5*(GradCurr>=0)*(xOld+right)+0.5*(GradCurr<0)*(xOld+left);
            Aux=xNew*E-Beta*E2;
            CtrAux=Aux-max(Aux);
            SumAux=sum(exp(CtrAux));
            ProbNew=exp(CtrAux)/SumAux;    % matrix Nbr x 1
            if (round(sum(ProbNew))~=1),
                disp(sprintf('Numerical problem for the Beta-update part'));
            end;
            GradFctObjNew=MeanQ-E*ProbNew';
            PlotGG(nn+1)=GradFctObjNew;

            % Stop or not the prospection
            %   based on the gradient close or not to zero.
            if (abs(GradFctObjNew)<=TolGrad | (nn==Nmax)),
                stop=1;
            else
                left=(GradCurr>0)*xOld+(GradCurr<0)*left;
                right=(GradCurr>=0)*right+(GradCurr<0)*xOld;
                GradCurr=GradFctObjNew;
                xOld=xNew;
                nn=nn+1;
            end;
        end;
    end;


    
%% 3- DISPLAY
%%%%%%%%%%%%%%
    figure(3);
    subplot(3,1,2);
    plot(ProbOld(1,:),'r:');
    hold on;
    plot(ProbNew(1,:),'g');
    title('weights');
%     subplot(3,1,3);
%     semilogy(abs(PlotGG(1:nn)));
%     title('gradient');
    drawnow
        
      
    
clear CtrAux Aux 

out.Beta=xNew;
out.DiagCvg=(nn<Nmax);
out.MeanQ=MeanQ;
out.GradWinit=E*ProbOld';
out.GradWend=E*ProbNew';
