    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  MCVEM : MONTE-CARLO VARIATIONAL EXPECTATION MAXIMIZATION
%%
%%  Algorithm proposed by F. FORBES and G. FORT (2005).
%%
%%  Codes developed by G. FORT : 
%%      matlab codes with built-in C functions.
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  function InterfaceMCVEM : 
%%          choose the parameters of the algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  The sections are filled with default value.
%%
%%  Sections : 
%%  1 : Test on the input variables and rename some variables
%%  2 : Parameters for the MCVEM loop
%%  3 : Initialisation of the iterative algorithm
%%  4 : Iterative procedure and Display.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  0- WELCOME !
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(sprintf('\t \t \t \t MONTE CARLO VARIATIONAL EM '));
disp(sprintf('\t \t \t \t by F. Forbes and G. Fort (2005)'));
disp(sprintf('\n Description of the different figures : '));
disp(sprintf('\t \t Fig 1. Observation, and evolution of the segmentation process.'));
disp(sprintf('\t \t Fig 2. [top] Follow the iterative procedure that solves the fixed point'));
disp(sprintf('\t \t \t  equation verified by the product distribution Q(z)= Prod_k q_k(z):'));
disp(sprintf('\t \t \t  at each pixel k, q_k(z) is a distribution on {1, ..., %d}.',G));
disp(sprintf('\t \t \t  We observe the evolution of q_k(1) along this iterative procedure.'));
disp(sprintf('\t \t Fig 2. [bottom] Follow the path of the Markov chain, produced at each iteration of MCVEM. '));
disp(sprintf('\t \t Fig 3. Control of the Beta-parameter update at each iteration of MCVEM.'));
disp(sprintf('\t \t \t [top] energy of each Markov chain sample'));
disp(sprintf('\t \t \t [bottom] weights in the computation of Grad(ln Tilde W)'));
disp(sprintf('\t \t \t of the dichotomy procedure.'));
disp(sprintf('\t \t Fig 4. Evolution of the parameter Beta as a function of the iterations of MCVEM.'));
disp(sprintf('\n These figures are defined in :  '));
disp(sprintf('\t \t Fig 1. see the file CodesMCVEM/InterfaceMCVEM.m '));
disp(sprintf('\t \t Fig 2. [top] see the file CodesMCVEM/RelaxationStep1.m '));
disp(sprintf('\t \t Fig 2. [bottom] see the file CodesMCVEM/SimulVarZ.m '));
disp(sprintf('\t \t Fig 3. [top] see the file CodesMCVEM/SimulVarZ.m '));
disp(sprintf('\t \t Fig 3. [center,bottom] see the file CodesMCVEM/Dichotomy.m '));
disp(sprintf('\t \t Fig 4. see the file CodesMCVEM/InterfaceMCVEM.m '));

wait=5;
disp(sprintf('\n  \n WAIT %d sec...',wait));
pause(wait)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  1 : TEST on the input values
%%      and
%%      RENAME some variables.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    %%  Test the dimension of the observed image
    if ((size(ObsPict,1)==1) | (size(ObsPict,2)==1))
        disp(sprintf('The observed image has to be in a matrix form'));
        return;
    end;
    
    %%  Test the dimension of the original image, if available.
    if ((PictAvailable==1) & ((size(TruePict,1)==1) | (size(TruePict,2)==1))),
        disp(sprintf('The original image has to be in a matrix form'));
        return;
    end;
    
    %%  Test if the two images have the same dimensions, when both available.
    if ((PictAvailable==1) & ((size(ObsPict,1)~=size(TruePict,1)) | (size(ObsPict,2)~=size(TruePict,2)))),
        disp(sprintf('The original image and the observed image must have the same size'));
        return;
    end;
     
    %%  Test the dimension of the initialization, if available.
    if ((InitAvailable==1) & ((size(InitPartition,1)==1) | (size(InitPartition,2)==1))),
        disp(sprintf('The initial partition has to be in a matrix form'));
        return;
    end;
    
     %%  Test the dimensions of the initial partition.
     if ((InitAvailable==1) & ((size(ObsPict,1)~=size(InitPartition,1)) | (size(ObsPict,2)~=size(InitPartition,2)))),
        disp(sprintf('The initial partition and the observed image must have the same size'));
        return;
    end;
    
    
    %%  Test on 'G', the number of classes
    if ((G<2) | (floor(G)~=G)),
        disp(sprintf('The number of classes, G, has to be an integer >=2'));
        return;
    end;
    
    %% Test a correct initialization file
    if (InitAvailable==1) 
        for gg=1:G,
            if length(find(InitPartition==gg))==0,
                disp(sprintf('Error in the initial partition : at least one class is missing'));
                return;
            end;
        end;
    end;
   
    %%  Test on 'NbrNeigh'
    if ((strcmp(NbrNeigh,'4-Neigh')==0) &  (strcmp(NbrNeigh,'8-Neigh')==0))
        disp(sprintf('For each site, the neighbors are the 4 or the 8 sites surrounding it.'));
        return;
    end;
     
    %%  Rename some variables.
    [I J]=size(ObsPict); % I = Nbr of rows and J = Nbr of columns.
    
    
    
    
    
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% 2 : PARAMETERS FOR THE MCVEM LOOPS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%  2-1 : Stopping criterion of the MCVEM loops
    %%      BASED   on the relative variation of the parameters lower
    %%              than a threshold.
    %%      AND     a maximal number of loops.
        %% Initialisation of the stopping criterion
        Stop=0; % Stop=1 iff the algorithm stops.
        %%  Maximal number of loops
        NbrIterMax=200;
        %% Threshold for the relative variation.
        Tol=0.0001;
      
     %% 2-2 : Solving the fixed-point equation by an iterative algorithm
        %% Initialisation of this procedure
        InitQdistMethod='LastIn'; % choose 'LastIn' or 'AllTheSame'.
        %% Maximal number of iterations
        NbrIterMaxQdist=2000;
        %%  Threshold for the relative variation of all the components of Q
        TolQdist=0.0001;
             
     %% 2-3 : Simulation step
        %%  Initialization of the Markov chain :
        InitChainMethod='LastIn'; % choose 'LastIn' or 'AllTheSame' or 'AtRandom'
        %%  Burn-in 
        BurnIn=0*ones(1,NbrIterMax);
        %%  Minimal length of the Markov chain :
        NbrSimulMin=100;
        %% Length of the Markov chain :
        NbrSimul=NbrSimulMin+ceil((2*(1:NbrIterMax)).^1.3);
        %% Algorithm for the importance sampling :
        MethSimul='PriorGibbsNonSyst'; % either 'PriorGibbsSyst' or 'PriorGibbsNonSyst' or 'PriorGibbsRandom'.
                                        % or 'PriorGibbsRandScan' if and ONLY if NbrNeigh='4-Neigh';
                                        
                                        
     %% 2-4 : Local optimization for the Beta-parameter.
        Gamma=0.005*ones(1,NbrIterMax);
        
     %% 2-5 :  Allocate the matrices in which the results are stocked.
        %% Successive values of the Beta-parameter.
        Beta=zeros(1,NbrIterMax);  % Successive values of the Beta-parameter.
        %% Successive values of the means.
        Mu=zeros(G,NbrIterMax);   
        %% Successive values of the variances.
        Sigma2=zeros(G,NbrIterMax);   
        %% Number of iterations when solving the fixed point equation for determining the Q-distribution.
        NbrIterQ=zeros(1,NbrIterMax); 
        %%  Diagnostic of convergence of this iterative computation for the Q-distribution.
        DiagCvgQ=zeros(1,NbrIterMax);  
        %% Diagnostic of convergence of the optimization procedure.
        DiagCvgNR=zeros(1,NbrIterMax);
        %   Expected mean of H under Q
        HQdist=zeros(1,NbrIterMax);
        % Nbr Errors in segmentation
        if PictAvailable==1,
            Error=zeros(1,NbrIterMax); 
        end;
        % Nbr of pixels that differ between two successive segmentations
        DeltaSegm=zeros(1,NbrIterMax);
     
   
             
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  3 : INITIALISATION OF THE iterative ALGORITHM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% 3-1 :  Parameter Beta 
        if strcmp(NbrNeigh,'4-Neigh')==1
            Beta(1)=1;
        else
            Beta(1)=0.5;
        end;
        
        BetaOld=Beta(1);
        
    %% 3-2 : Parameters Mu and Sigma2
    %% initialization based on the initial partition if given, on k-means
    %% otherwise.
        if InitAvailable==1
            PictCol=reshape(ObsPict,I*J,1);  
            Partition=reshape(InitPartition,I*J,1);
        else
            PictCol=reshape(ObsPict,I*J,1);    % matrix IJ x 1
            Partition=kmeans(PictCol,G);    % matrix IJ x 1
        end;
        %% Determine Mean and Variances of each group
        for gg=1:G,
            ThetaOld(gg,1)=mean(PictCol(Partition==gg));    % matrix G x 1, for the "Mean"
            ThetaOld(gg,2)=var(PictCol(Partition==gg)); % matrix G x 1, for the "Variance"
        end;
        %% Sort the classes in the ascending order of the initial "Mean"
        [ThetaOld xx]=sort(ThetaOld);
        PartitionBis=zeros(I*J,1);
        for gg=1:G,
            PartitionBis=gg*(Partition==xx(gg))+PartitionBis;
        end
        Partition=PartitionBis;
        %% Stock the initial values of the parameter
        Mu(:,1)=ThetaOld(:,1);
        Sigma2(:,1)=ThetaOld(:,2);

        
     %% 3-3 : Initialization of the procedure that solves the fixed-point
     %% equation
        %% Based  on the given initial partition, or on k-means otherwise.
        Qinit=reshape(Partition,I,J);   % matrix I x J, {1, ..., G}-valued.
        QdistInit=zeros(I,J,G);
        for gg=1:G,
            QdistInit(:,:,gg)=(Qinit==gg);   % matrix I x J x G, Q distribution at each site (i,j).
        end;
        SegmentationInit=Qinit; % matrix I x J
            
     %% 3-4 : Initialization of the chain
        Zinit=ceil(rand(I,J)*G);    % matrix I x J, {1,...,G}-valued.
        AuxChain=SimulVarZ(Zinit,Beta(1),G,10,0,NbrNeigh,Qinit,MethSimul,ObsPict,Mu(:,1),Sigma2(:,1));
        Zinit=AuxChain.Zfinal;
        
    %% 3-5 : Initialization of the number of loops
        nn=1;
    
    %%  3-6 : Initialization of the segmentation 
        OldSegm=SegmentationInit;    
    
    %%  3-6 : Display
        disp(sprintf('Initialisation :'));
        %   Display the initial values
        disp(sprintf('\t \t Mean %f',Mu(:,1)));
        disp(sprintf('\t \t Std Dev %f',sqrt(Sigma2(:,1))));
        disp(sprintf('\t \t Beta %f',Beta(1)));
        % %% Display the original image, the observation and the initial partition.
        total=3+(PictAvailable==1);
        figure(1);
        clf;
        subplot(1,total,1);
        Image(ObsPict);
        title('Observation')
        subplot(1,total,2);
        Image(SegmentationInit);
        title('Initial Segm.')
        subplot(1,total,3);
        Image(SegmentationInit);
        title('Current Segm.')
        if total>3
            subplot(1,total,4)
            Image(TruePict);
            title('Original image')
        end;    
        drawnow
   
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  4 : ITERATIVE PROCEDURE and DISPLAY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    while Stop==0,  
        %%  4-1 : Update the Q-approximation of the posterior density
            %% Run the iterative algorithm
            outRel=RelaxationStep1(ObsPict,QdistInit,ThetaOld,BetaOld,NbrNeigh,NbrIterMaxQdist,TolQdist);
            QdistNew=outRel.M; % matrix I x J x G, distribution at each site (i,j).
            
            %%  Display the report
            disp(sprintf('Iteration %f',nn));
            disp(sprintf('\t  Step 1 : Determine the Q-distribution'));
            disp(sprintf('\t \t   Nbr of iteration %f',outRel.NbrIter));
            disp(sprintf('\t \t   Control :'));
            disp(sprintf('\t \t \t Diag of cvgce (1 if OK) %f',outRel.DiagCvg));
            
        %%  4-2 : Update Means and Variances
            for gg=1:G,
                ThetaNew(gg,1)=sum(sum(ObsPict.*QdistNew(:,:,gg)))/(sum(sum(QdistNew(:,:,gg))));
                DD=(ObsPict-ThetaNew(gg,1)).^2;
                ThetaNew(gg,2)=sum(sum(DD.*QdistNew(:,:,gg)))/(sum(sum(QdistNew(:,:,gg))));
            end;
            disp(sprintf('\t Step 2a : Update Mean and Variances'));
            disp(sprintf('\t \t   Mean %f',ThetaNew(:,1)));
            disp(sprintf('\t \t   Std Dev %f',sqrt(ThetaNew(:,2))));
     
        %%  4-3 : Update Beta
            %% Simulate the chains and return the successive energy
            outSimul=SimulVarZ(Zinit,BetaOld,G,NbrSimul(nn),BurnIn(nn),NbrNeigh,QdistNew,MethSimul,ObsPict,Mu(:,nn),Sigma2(:,nn));    % structure with two fields
            Zinit=outSimul.Zinit;
            %% Compute the maximum
            outNR=Dichotomy(QdistNew,outSimul.Energy,outSimul.Energy2,BetaOld,NbrNeigh,Gamma(nn));
            BetaNew=outNR.Beta;
            %%  Display
            disp(sprintf('\t Step 2b : Update Beta'));
            disp(sprintf('\t \t   Beta %f',BetaNew));
            disp(sprintf('\t \t   Nbr Simul (and burn in : %d) : %d',BurnIn(nn),NbrSimul(nn)));
            disp(sprintf('\t \t   Control :'));
            disp(sprintf('\t \t \t Cvgce of Dichotomy (1 if OK) : %d',outNR.DiagCvg));
            disp(sprintf('\t \t \t -(GradLnW) init : %f',outNR.GradWinit));
            disp(sprintf('\t \t \t -(GradLnW) end : %f',outNR.GradWend));
            disp(sprintf('\t \t \t - (MeanH) under Q : %f',outNR.MeanQ));
            
        
        %%  4-4 : Stop or not the iteration
            Var1=max(max(abs((ThetaNew-ThetaOld)./ThetaOld)));
            Var2=abs((BetaNew-BetaOld)./BetaOld);
            Var=max([Var1 Var2]);
            if (nn==NbrIterMax)     %((Var<=Tol) | (nn==NbrIterMax)) 
                Stop=1;
            else
                nn=nn+1;  
            end;
       
        %%  4-5 : Stock the results
            NbrIterQ(nn)=outRel.NbrIter;
            DiagCvgQ(nn)=outRel.DiagCvg;
            DiagCvgNR(nn)=outNR.DiagCvg;
            Mu(:,nn)=ThetaNew(:,1);
            Sigma2(:,nn)=ThetaNew(:,2);
            Beta(nn)=BetaNew;
            HQdist(nn)=outNR.MeanQ;
            
        %%  4-6 : Update some variables
            %% Initialisation QdistInit
            if strcmp(InitQdistMethod,'LastIn')==1,
                QdistInit=QdistNew;
            else
                for gg=1:G,
                    QdistInit(:,:,gg)=(Qinit==gg);   % matrix I x J x G, Q distribution at each site (i,j).
                end;
            end;
            %%  Initialisation of the chain
            if strcmp(InitChainMethod,'LastIn')==1,
                Zinit=outSimul.Zfinal;
            end;
            if strcmp(InitChainMethod,'AtRandom')==1,
                bb=Beta(nn)-0.3+rand*(0.3);
                AuxChain=SimulVarZ(Zinit,bb,G,ceil(NbrSimul(nn)/2),0,NbrNeigh,QdistNew,MethSimul,ObsPict,Mu(:,nn),Sigma2(:,nn));
                Zinit=AuxChain.Zfinal;
            end;
            
            
            
            %% The 'old' values of the parameter
            ThetaOld=ThetaNew;
            BetaOld=BetaNew;
        
        %% 4-7 : Restaure the missing data 
        [x HatZnew]=max(QdistNew,[],3);
        HatZ=HatZnew;
        DeltaSegm(nn)=sum(sum(HatZ~=OldSegm));
        disp(sprintf('\t  Step 3 : Segmentation'));
        disp(sprintf('\t \t  Variation in the segmentation : %d / %d (%2.3f)',DeltaSegm(nn),I*J,DeltaSegm(nn)*100/(I*J)));
        OldSegm=HatZ;
        if PictAvailable==1
            Error(nn)=sum(sum(HatZ~=TruePict));
            disp(sprintf('\t  \t  Nbr errors in segmentation : %d / %d (%2.3f) ',Error(nn),I*J,Error(nn)*100/(I*J)));     
        end;
        
        
        
        
        %%  4-8 : Display 
            %%  Beta every 10 iterations.
            figure(4);
            if (nn/10)==round(nn/10),
                plot(Beta(1:nn));
            end;
            title('Parameter Beta');
            drawnow
        
            %%  The pictures
            total=3+(PictAvailable==1);
            figure(1);
            clf;
            subplot(1,total,1);
            Image(ObsPict);
            title('Observation')
            subplot(1,total,2);
            Image(SegmentationInit);
            title('Initial Segm.')
            subplot(1,total,3);
            Image(HatZ);
            title('Current Segm.')
            if total>3
                subplot(1,total,4)
                Image(TruePict);
                title('Original image')
            end;    
            drawnow
    
    end;    % w.r.t. the "while"

    
    
    
    
    
    
    
    
    

       


 